import logging
from enum import Enum


class AuditLogAction(Enum):
    CREATE = "create"
    UPDATE = "update"
    DELETE = "delete"
    ACCESS = "access"


class ChangeScheduleAction(Enum):
    DEPLOY = "deploy"
    DECOM = "decom"
    SHARED_DATA_UPDATE = "shared_data_update"


class ChangeScheduleInterval(Enum):
    YEARLY = 0
    MONTHLY = 1
    WEEKLY = 2
    DAILY = 3
    HOURLY = 4
    MINUTELY = 5


class ChangeScheduleStatus(Enum):
    RUNNING = "running"
    QUEUED = "queued"
    DONE = "done"


class ChangeScheduleExecutionStyle(Enum):
    SEQUENTIAL = "sequential"
    PARALLEL = "parallel"


class CryptoType(Enum):
    PLAIN_TEXT = "PT"
    ANSIBLE_VAULT = "AV"
    FERNET = "FE"


class ExtApiAuthType(Enum):
    BASIC = "Basic"
    BEARER = "Bearer"
    HEADER = "Header"


class InfluxRetention(Enum):
    KEEP_1_HOUR = "1H"
    KEEP_1_DAY = "1D"
    KEEP_7_DAYS = "7D"
    KEEP_1_MONTH = "1M"
    KEEP_3_MONTHS = "3M"
    KEEP_6_MONTHS = "6M"
    KEEP_1_YEAR = "1Y"


class LogLevel(str, Enum):
    DEBUG = "DG"
    INFO = "IN"
    INFORMATIONAL = "IN"
    WARNING = "WA"
    ERROR = "ER"

    def to_native(self) -> int:
        mapping: dict[str, int] = {
            self.DEBUG: logging.DEBUG,
            self.INFO: logging.INFO,
            self.INFORMATIONAL: logging.INFO,
            self.WARNING: logging.WARNING,
            self.ERROR: logging.ERROR,
        }

        return mapping.get(self, logging.INFO)


class ModuleType(Enum):
    AWX = "awx-based"


class PhaseStatus(Enum):
    DEPLOYED = "DE"
    ERRORED = "ER"
    ORDERED = "OR"
    PENDING_UPDATE = "PU"
    READY = "RA"
    RETIRED = "RE"
    RETIRING = "RI"
    UPDATING = "UP"
    POLLING = "PO"


class PhaseType(Enum):
    EXT_API_PHASE = "ExtApiPhase"
    SERVICE_MANAGEMENT_PHASE = "ServiceManagementPhase"
    POLLING_PHASE = "PollingPhase"
    PYTHON_SCRIPT_PHASE = "PythonScriptPhase"


class PhaseInstanceAction(Enum):
    NONE = "NONE"
    RUNNING = "RUNNING"
    RERUN = "RERUN"
    CONTINUE = "CONTINUE"


class ReportCategory(Enum):
    DEFAULT = "default"
    SYSTEM = "system"


class ServiceCategory(Enum):
    DEFAULT = "default"
    SYSTEM = "system"


class TagColor(Enum):
    RED = "#F44336"
    PINK = "#E91E63"
    PURPLE = "#9C27B0"
    DEEP_PURPLE = "#673AB7"
    INDIGO = "#3F51B5"
    BLUE = "#2196F3"
    LIGHT_BLUE = "#03A9F4"
    CYAN = "#00BCD4"
    TEAL = "#009688"
    GREEN = "#4CAF50"
    LIGHT_GREEN = "#8BC34A"
    LIME = "#CDDC39"
    YELLOW = "#FFEB3B"
    AMBER = "#FFC107"
    ORANGE = "#FF9800"


class UploadFileCategory(Enum):
    REGULAR = "regular"
    PYTHON_SCRIPT = "python-script"
    PYTHON_DEPENDENCIES = "python-dependencies"
    SYSTEM = "system"



# @generated
# This file was automatically generated by the ncae-sdk code generator.
# DO NOT EDIT MANUALLY, ANY CHANGES WILL BE OVERWRITTEN.

from typing import Any, Final, Iterator, Optional

from typing_extensions import Unpack

from ncae_sdk._resource import ResourceId
from ncae_sdk._sync.endpoint import Endpoint
from ncae_sdk._util import map_sync
from ncae_sdk.resources._schema import (
    User,
    UserCreate,
    UserCreateModel,
    UserFilter,
    UserFilterModel,
    UserUpdate,
    UserUpdateModel,
)


class UserEndpoint(Endpoint):
    BASE_PATH: Final[str] = "admin/v1/user"

    def list(self, **filters: Unpack[UserFilter]) -> Iterator[User]:
        results = self._list(self.BASE_PATH, UserFilterModel, filters)
        return map_sync(User.parse_api, results)

    def find(self, **filters: Unpack[UserFilter]) -> Optional[User]:
        result = self._get_by_filters(self.BASE_PATH, UserFilterModel, filters)
        return User.parse_api(result) if result else None

    def get(self, rid: ResourceId) -> Optional[User]:
        result = self._get_by_id(self.BASE_PATH, rid)
        return User.parse_api(result) if result else None

    def create(self, **payload: Unpack[UserCreate]) -> User:
        result = self._create(self.BASE_PATH, UserCreateModel, payload, preprocessor=self._preprocessor)
        return User.parse_api(result)

    def update(self, rid: ResourceId, /, **payload: Unpack[UserUpdate]) -> User:
        result = self._update(self.BASE_PATH, rid, UserUpdateModel, payload, preprocessor=self._preprocessor)
        return User.parse_api(result)

    def set_password(self, rid: ResourceId, password: str) -> None:
        self._session.post(
            f"{self.BASE_PATH}/{rid}/set_password",
            json={"password": password, "repeat_password": password},
        )

    @staticmethod
    def _preprocessor(value: dict[str, Any]) -> dict[str, Any]:
        if password := value.get("password"):
            value["repeat_password"] = password
        if become_password := value.get("become_password"):
            value["repeat_become_password"] = become_password

        return value



from collections.abc import AsyncIterator
from typing import Any, Final, Optional

from typing_extensions import Unpack

from ncae_sdk._async.endpoint import AsyncEndpoint
from ncae_sdk._resource import ResourceId
from ncae_sdk._util import map_async
from ncae_sdk.resources._schema import (
    Credential,
    CredentialCreate,
    CredentialCreateModel,
    CredentialFilter,
    CredentialFilterModel,
    CredentialUpdate,
    CredentialUpdateModel,
)


class AsyncCredentialEndpoint(AsyncEndpoint):
    BASE_PATH: Final[str] = "dashboard/v1/credential"

    async def list(self, **filters: Unpack[CredentialFilter]) -> AsyncIterator[Credential]:
        results = self._list(self.BASE_PATH, CredentialFilterModel, filters)
        return map_async(Credential.parse_api, results)

    async def find(self, **filters: Unpack[CredentialFilter]) -> Optional[Credential]:
        result = await self._get_by_filters(self.BASE_PATH, CredentialFilterModel, filters)
        return Credential.parse_api(result) if result else None

    async def get(self, rid: ResourceId) -> Optional[Credential]:
        result = await self._get_by_id(self.BASE_PATH, rid)
        return Credential.parse_api(result) if result else None

    async def create(self, **payload: Unpack[CredentialCreate]) -> Credential:
        result = await self._create(
            self.BASE_PATH, CredentialCreateModel, payload, preprocessor=self._add_repeat_passwords
        )
        return Credential.parse_api(result)

    async def update(self, rid: ResourceId, /, **payload: Unpack[CredentialUpdate]) -> Credential:
        result = await self._update(
            self.BASE_PATH, rid, CredentialUpdateModel, payload, preprocessor=self._add_repeat_passwords
        )
        return Credential.parse_api(result)

    async def delete(self, rid: ResourceId) -> bool:
        return await self._delete(self.BASE_PATH, rid)

    @staticmethod
    def _add_repeat_passwords(value: dict[str, Any]) -> dict[str, Any]:
        if password := value.get("password"):
            value["repeat_password"] = password
        if become_password := value.get("become_password"):
            value["repeat_become_password"] = become_password

        return value



from ncae_sdk.types._enums import (
    AuditLogAction,
    ChangeScheduleAction,
    ChangeScheduleExecutionStyle,
    ChangeScheduleInterval,
    ChangeScheduleStatus,
    CryptoType,
    ExtApiAuthType,
    InfluxRetention,
    LogLevel,
    ModuleType,
    PhaseInstanceAction,
    PhaseStatus,
    PhaseType,
    ReportCategory,
    ServiceCategory,
    TagColor,
    UploadFileCategory,
)
from ncae_sdk.types._fields import CmdbOptionalStr, CmdbTextList
from ncae_sdk.types._models import (
    AuditLogChange,
    AwxInventoryItem,
    InfluxDataPoint,
    InfluxTagFilter,
)
from ncae_sdk.types._types import ServiceConfig

__all__ = [
    "AuditLogAction",
    "AuditLogChange",
    "AwxInventoryItem",
    "ChangeScheduleAction",
    "ChangeScheduleExecutionStyle",
    "ChangeScheduleInterval",
    "ChangeScheduleStatus",
    "CmdbOptionalStr",
    "CmdbTextList",
    "CryptoType",
    "ExtApiAuthType",
    "InfluxDataPoint",
    "InfluxRetention",
    "InfluxTagFilter",
    "LogLevel",
    "ModuleType",
    "PhaseStatus",
    "PhaseType",
    "PhaseInstanceAction",
    "ReportCategory",
    "ServiceCategory",
    "ServiceConfig",
    "TagColor",
    "UploadFileCategory",
]



from collections.abc import AsyncIterator
from typing import Final, Optional

from typing_extensions import Unpack

from ncae_sdk._async.endpoint import AsyncEndpoint
from ncae_sdk._resource import ResourceId
from ncae_sdk._util import map_async
from ncae_sdk.resources._schema import (
    AwxModule,
    AwxModuleCreate,
    AwxModuleCreateModel,
    AwxModuleFilter,
    AwxModuleFilterModel,
    AwxModuleUpdate,
    AwxModuleUpdateModel,
)
from ncae_sdk.types import AwxInventoryItem


class AsyncAwxModuleEndpoint(AsyncEndpoint):
    BASE_PATH: Final[str] = "admin/v1/awx-based-module"

    async def list(self, **filters: Unpack[AwxModuleFilter]) -> AsyncIterator[AwxModule]:
        results = self._list(self.BASE_PATH, AwxModuleFilterModel, filters)
        return map_async(AwxModule.parse_api, results)

    async def find(self, **filters: Unpack[AwxModuleFilter]) -> Optional[AwxModule]:
        result = await self._get_by_filters(self.BASE_PATH, AwxModuleFilterModel, filters)
        return AwxModule.parse_api(result) if result else None

    async def get(self, rid: ResourceId) -> Optional[AwxModule]:
        result = await self._get_by_id(self.BASE_PATH, rid)
        return AwxModule.parse_api(result) if result else None

    async def create(self, **payload: Unpack[AwxModuleCreate]) -> AwxModule:
        result = await self._create(self.BASE_PATH, AwxModuleCreateModel, payload)
        return AwxModule.parse_api(result)

    async def update(self, rid: ResourceId, /, **payload: Unpack[AwxModuleUpdate]) -> AwxModule:
        result = await self._update(self.BASE_PATH, rid, AwxModuleUpdateModel, payload)
        return AwxModule.parse_api(result)

    async def list_inventory(self, rid: ResourceId) -> AsyncIterator[AwxInventoryItem]:
        results = self._paginate(f"{self.BASE_PATH}/{rid}/inventory")
        return map_async(AwxInventoryItem.model_validate, results)



from ncae_sdk.fastapi.extapi._context import ExtApiPhaseContext, ExtApiReportContext
from ncae_sdk.fastapi.extapi._contracts import (
    CallbackT,
    CmdbT,
    ExtApiCredential,
    ExtApiDevice,
    ExtApiDeviceModel,
    ExtApiPhaseExtraVars,
    ExtApiPhaseRequest,
    ExtApiReportField,
    ExtApiReportRequest,
    ExtApiReportResponse,
    QueryT,
)
from ncae_sdk.fastapi.extapi._decorators import extapi_phase_route, extapi_report_route

__all__ = [
    "CallbackT",
    "CmdbT",
    "ExtApiCredential",
    "ExtApiDevice",
    "ExtApiDeviceModel",
    "ExtApiPhaseContext",
    "ExtApiPhaseExtraVars",
    "ExtApiPhaseRequest",
    "ExtApiReportContext",
    "ExtApiReportField",
    "ExtApiReportRequest",
    "ExtApiReportResponse",
    "QueryT",
    "extapi_phase_route",
    "extapi_report_route",
]
